"""
Message处理模块
提供消息格式化、准备和验证功能
"""

from .builder import prepare_messages, rebuild_messages_single_image, convert_images_to_base64, prepare_request_data
from .formatter import convert_to_json, extract_field

__all__ = [
    'prepare_messages',
    'rebuild_messages_single_image',
    'convert_images_to_base64',
    'prepare_request_data',
    'convert_to_json',
    'extract_field',
]



"""
重试配置模块
定义所有与重试相关的关键词配置
"""

# 图片下载相关的错误关键词列表
IMAGE_DOWNLOAD_ERROR_KEYWORDS = [
    "Download the media resource timed out",
    "Unable to download the media",  # modelscope
    "Failed to download multimodal content",
    "Download multimodal file timed out",
    "Invalid image data",
]
MIN_LIMIT_ERROR_KEYWORDS = [
    "Rate limit exceeded: free-models-per-min",  # openrouter
    "is temporarily rate-limited upstream",  # openrouter
]

# 默认的重试关键词列表
DEFAULT_RETRY_KEYWORDS = [
    "Too many requests",
    "频繁操作，请稍后重试",  # gitcode
    "Request rate increased too quickly",  # modelscope
    "模型当前访问量过大",  # zhipu
    "Allocated quota exceeded, please increase your quota limit",
    "Max retries exceeded with url",
    "Requests rate limit exceeded, please try again later",
    "Free credits temporarily have rate limits",  # vercel
    "experiencing high traffic right now! Please try again soon",  # cerebras
]

DEFAULT_RETRY_KEYWORDS.extend(MIN_LIMIT_ERROR_KEYWORDS)
DEFAULT_RETRY_KEYWORDS.extend(IMAGE_DOWNLOAD_ERROR_KEYWORDS)

# API限流相关的重试关键词列表
DEFAULT_RETRY_API_KEYWORDS = [
    "Request limit exceeded",  # modelscope
    "You have exceeded today",  # modelscope
    "We have to rate limit you",  # modelscope
    "make sure your associated Aliyun account is real-name verified",  # modelscope
    "The free tier of the model has been exhausted",  # dashscope
    "Rate limit exceeded: free-models-per-day-high-balance",  # openrouter
    "API key USD spend limit exceeded",  # openrouter
    "You exceeded your current quota",  # gemini
    "You have exceeded your monthly included credits",  # huggingface
    "You have depleted your monthly included credits",  # huggingface
    "Tokens per day limit exceeded",  # cerebras_openai
    "on tokens per day (TPD)",  # groq
    "免费使用额度已用完",  # gitcode , modelscope
    "您的账户已达到速率限制",  # zhipu
    # 模型不存在
    "does not exist or you do not have access to it",  # cerebras_openai
    "no provider supported",  # modelscope
    "Invalid model id",  # gitcode
    "Inference Serverless API for this model not found",  # gitcode
]



"""
验证程序模块
包含HTML验证、字符串验证、值验证等功能
"""

from .html_validator import check_allowed_tags
from .string_validator import contains_chinese, remove_chinese, contains_special_symbols, remove_special_symbols
from .value_validator import validate_dict, validate_string

__all__ = [
    'check_allowed_tags',
    'contains_chinese',
    'remove_chinese',
    'contains_special_symbols',
    'remove_special_symbols',
    'validate_dict',
    'validate_string',
]



"""
图片Base64缓存管理器
使用LRU策略缓存最多10张图片的base64编码
"""

from collections import OrderedDict
from typing import Optional


class ImageBase64Cache:
    """
    图片Base64缓存管理器
    使用LRU（最近最少使用）策略管理缓存
    """

    def __init__(self, max_size: int = 10):
        """
        初始化缓存

        Args:
            max_size: 最大缓存数量，默认10张图片
        """
        self.max_size = max_size
        self.cache = OrderedDict()  # 使用OrderedDict实现LRU

    def get(self, url: str) -> Optional[str]:
        """
        从缓存中获取base64字符串

        Args:
            url: 图片URL

        Returns:
            base64字符串，如果不存在则返回None
        """
        if url not in self.cache:
            return None

        # 移动到末尾（标记为最近使用）
        self.cache.move_to_end(url)
        return self.cache[url]

    def put(self, url: str, base64_str: str) -> None:
        """
        将图片base64存入缓存

        Args:
            url: 图片URL
            base64_str: base64编码字符串
        """
        if url in self.cache:
            # 如果已存在，移动到末尾
            self.cache.move_to_end(url)
            self.cache[url] = base64_str
        else:
            # 新增条目
            self.cache[url] = base64_str
            # 如果超过容量，删除最旧的条目
            if len(self.cache) > self.max_size:
                self.cache.popitem(last=False)

    def clear(self) -> None:
        """清空缓存"""
        self.cache.clear()

    def size(self) -> int:
        """返回当前缓存大小"""
        return len(self.cache)

    def contains(self, url: str) -> bool:
        """检查URL是否在缓存中"""
        return url in self.cache



from .kit import *
from .validators.string_validator import (
    contains_chinese,
    remove_chinese,
    contains_special_symbols,
    remove_special_symbols,
)
from .validators.html_validator import validate_html, validate_html_fix
from .kits.title_kit import check_title, shorten_title, generate_title
from .kits.des_kit import generate_html
from .kits.attribute_kit import fill_attr
from .kits.cat_kit import predict_cat_direct, predict_cat_gradual

__all__ = [
    "translate_options",
    "contains_chinese",
    "remove_chinese",
    "contains_special_symbols",
    "remove_special_symbols",
    "validate_html",
    "validate_html_fix",
    "check_title",
    "shorten_title",
    "generate_title",
    "generate_html",
    "fill_attr",
    "predict_cat_direct",
    "predict_cat_gradual",
]



from typing import Any, Dict, List, Union


def validate_dict(choices: List[Dict[str, Any]], search_data: Dict[str, Any]) -> Dict[str, Any]:
    """
    验证并查找指定值是否存在于字典列表中

    :param choices: (list[dict]): 要搜索的字典列表
    :param search_data: dict, 键名应与choices中字典的键名一致
    :return: dict, 匹配到的项，包含与search_data相同的键
    """
    if not isinstance(choices, list) or not all(isinstance(item, dict) for item in choices):
        raise TypeError("choices 必须是字典组成的列表")

    if not choices:
        raise ValueError("choices 不能为空列表")

    # 获取choices中第一个字典的键名
    first_item_keys = set(choices[0].keys())
    search_data_keys = set(search_data.keys())

    # 检查search_data的键是否与choices中字典的键一致
    if not search_data_keys.issubset(first_item_keys):
        raise ValueError(f"search_data的键 {search_data_keys} 必须是choices中字典键 {first_item_keys} 的子集")

    # 对 search_data_keys 进行排序，包含了字符 "id" 或 "value" 的键放在前面（这样可以优先匹配）
    search_data_keys = sorted(search_data_keys, key=lambda x: (x.find("id") != -1, x.find("value") != -1))

    # 在choices中查找匹配项
    for item in choices:
        for search_data_key in search_data_keys:
            if item[search_data_key] == search_data[search_data_key]:
                return item

    # 未找到匹配项
    print(f"未找到匹配项: {search_data}")
    if len(choices) < 30:
        print(f"choices: {choices}")

    return {}


def validate_string(choices: List[str], search_value: str):
    """
    验证并查找指定字符串是否存在于字符串列表中

    :param choices: List[str], 要搜索的字符串列表
    :param search_value: str, 要查找的字符串值
    :return: str, 匹配到的字符串值
    """

    if search_value in choices:
        return search_value

    # 未找到匹配项
    print(f"未找到匹配项: {search_value}")
    if len(choices) < 30:
        print(f"choices: {choices}")
    return None


def auto_validate(choices: List[Any], search_data: Any) -> Union[Dict[str, Any], str, None]:
    """
    根据choices的第一个元素类型自动选择验证程序

    Args:
        choices: 要搜索的列表，可以是字典列表或字符串列表
        search_data: 要查找的数据，字典或字符串

    Returns:
        匹配到的值：字典验证返回Dict，字符串验证返回str，未找到返回None
    """
    if not choices:
        print("choices 为空列表")
        return None

    # 根据列表第一个元素的类型选择验证器
    first_item = choices[0]

    if isinstance(first_item, dict):
        # 字典类型验证
        if not isinstance(search_data, dict):
            print(f"字典类型验证需要search_data为字典，实际类型: {type(search_data)}")
            return None
        return validate_dict(choices, search_data)

    elif isinstance(first_item, str):
        # 字符串类型验证
        if not isinstance(search_data, str):
            print(f"字符串类型验证需要search_data为字符串，实际类型: {type(search_data)}")
            return None
        return validate_string(choices, search_data)

    else:
        print(f"不支持的列表元素类型: {type(first_item)}")
        return None



"""
消息构建器
负责根据不同提供商的要求构建消息格式
"""

from typing import List, Optional, Dict, Any, Tuple
from filekits.base_io import download_encode_base64, batch_download_encode_base64


def prepare_messages(
    provider_name: str,
    system_prompt: str,
    user_text: str,
    include_img: bool = False,
    img_list: Optional[List[str]] = None,
) -> List[Dict[str, Any]]:
    """
    根据提供商名称准备消息格式

    Args:
        provider_name: 提供商名称 ('dashscope', 'zhipu', 'openai', 'modelscope', 'ollama')
        system_prompt: 系统提示词
        user_text: 用户文本
        include_img: 是否包含图片
        img_list: 图片URL列表

    Returns:
        格式化后的消息列表
    """
    if img_list is None:
        if include_img:
            raise ValueError("包含图片的消息，图片 img_list 不能为空。")
        img_list = []

    if include_img and len(img_list) < 1:
        raise ValueError("包含图片的消息，图片 img_list 不能为空。")

    # 根据提供商构建不同格式的消息
    system_content, user_content = _build_content_by_provider(
        provider_name, system_prompt, user_text, include_img, img_list
    )

    # 构建消息结构
    user_message = {"role": "user", "content": user_content}

    if system_content:
        system_message = {"role": "system", "content": system_content}
        return [system_message, user_message]
    else:
        return [user_message]


def rebuild_messages_single_image(
    provider_name: str,
    system_prompt: str,
    user_text: str,
    reject_single_image: bool,
    img_list: list,
) -> List[Dict[str, Any]]:
    """
    重新构造messages，只使用第一张图片

    Args:
        provider_name: 提供商名称
        system_prompt: 系统提示词
        user_text: 用户文本
        reject_single_image: 是否禁止单张图片（为True时若图片数量为1则抛异常）
        img_list: 图片URL列表

    Returns:
        格式化后的消息列表（只包含第一张图片）
    """
    if not img_list:
        raise ValueError("图片 img_list 不能为空!")

    img_num = len(img_list)

    if reject_single_image:
        if img_num == 1:
            error_message = "异常：图片数量 == 1，未超出限制"
            print(f"{error_message}，img_list : {img_list}")
            raise Exception(error_message)

    # 重新构造messages，只使用第一张图片
    return prepare_messages(provider_name, system_prompt, user_text, True, [img_list[0]] if img_list else [])


def _build_content_by_provider(
    provider_name: str,
    system_prompt: str,
    user_text: str,
    include_img: bool,
    img_list: List[str],
    image_cache=None,  # 图片缓存参数
) -> tuple:
    """根据提供商构建内容格式"""

    if not include_img:
        return system_prompt, user_text

    if provider_name == "dashscope":
        user_content = [{"image": img} for img in img_list]
        user_content.append({"text": user_text})
        if system_prompt:
            system_content = [{"text": system_prompt}]
        else:
            system_content = []

    elif provider_name == "ollama":
        if system_prompt:
            system_content = system_prompt
        else:
            system_content = []
        user_content = user_text
        try:
            # todu 这里后续需要修改
            img_list = batch_download_encode_base64(img_list)
        except Exception as e:
            print(f"Ollama图片批量转换base64失败: {e}")
            raise Exception(f"图片下载或转换base64失败，url：{img_list}")

    # 兼容通用的 "openai", "modelscope", "openrouter" 格式 , 不支持 zhipu ( 可切换为 zhipu_openai 进行兼容 )
    else:
        if provider_name in ["openrouter", "gemini", "vercel", "github"]:
            # openrouter 需要base64格式的图片
            try:
                img_list = convert_images_to_base64(img_list, image_cache)  # 传递缓存
            except Exception as e:
                print(f"图片列表转base64失败: {e}")
                raise Exception(f"图片列表转base64失败，img_list：{img_list}")

        user_content = [{"type": "image_url", "image_url": {"url": img}} for img in img_list]
        if provider_name in ["gitcode"]:
            if system_prompt:
                system_prompt_user = f"# 任务角色与设定 \n{system_prompt}\n"
            else:
                system_prompt_user = ""
            user_prompt = f"# 任务相关信息 \n{user_text}\n"
            user_content.append({"type": "text", "text": system_prompt_user + user_prompt})
            system_content = []
        else:
            user_content.append({"type": "text", "text": user_text})
            if system_prompt:
                system_content = [{"type": "text", "text": system_prompt}]
            else:
                system_content = []

    return system_content, user_content


def convert_images_to_base64(img_list: List[str], image_cache=None) -> List[str]:
    """
    将图片URL列表转换为base64格式

    Args:
        img_list: 图片URL列表

    Returns:
        转换后的图片URL列表（仅保留转换成功的base64格式）
        注意：转换后的图片列表，不支持 sdk/platform/provider = zhipu ,
            如需使用，请使用名称 zhipu_openai 兼容 openai的格式
    """
    if not img_list:
        raise ValueError("图片 img_list 不能为空!")

    # 如果没有传入缓存对象，尝试从dispatcher获取全局缓存
    if image_cache is None:
        try:
            # 注意：必须在这里导入，避免循环引用
            # dispatcher.py 导入 message.convert_to_json，而 message.builder 又导入 dispatcher.ModelDispatcher
            # 如果在模块级别导入会造成循环引用错误
            from ..dispatcher import ModelDispatcher

            image_cache = ModelDispatcher.get_image_cache()
        except ImportError:
            # 如果无法导入，不使用缓存
            image_cache = None

    processed_img_list = []
    valid_extensions = ('.jpg', '.jpeg', '.png')
    successful_conversions = 0  # 成功转换的图片数量
    total_valid_images = 0  # 需要转换的有效图片数量

    for img_url in img_list:
        # 检查是否已经是base64格式
        if img_url.startswith('data:image/') and ';base64,' in img_url:
            # 已经是base64格式，直接添加到结果列表
            processed_img_list.append(img_url)
            # print(f"图片已经是base64格式，无需转换: {img_url}...")
            continue

        # 检查是否为有效的图片URL
        img_lower = img_url.lower()
        if img_lower.endswith(valid_extensions):
            total_valid_images += 1

            # 尝试从缓存中获取
            cached_base64 = None
            if image_cache is not None:
                cached_base64 = image_cache.get(img_url)
                if cached_base64:
                    print(f"已从缓存中获取图片base64: {img_url}")
                    # 构建完整的base64 URL
                    mime_type = 'image/jpeg' if img_lower.endswith(('.jpg', '.jpeg')) else 'image/png'
                    base64_url = f"data:{mime_type};base64,{cached_base64}"
                    processed_img_list.append(base64_url)
                    successful_conversions += 1
                    continue

            try:
                # 下载并转换为base64
                base64_str = download_encode_base64(img_url)
                if base64_str:
                    # 存入缓存
                    if image_cache is not None:
                        image_cache.put(img_url, base64_str)
                        # print(f"已将图片base64存入缓存: {img_url}")
                    # 构建base64格式的图片URL
                    mime_type = 'image/jpeg' if img_lower.endswith(('.jpg', '.jpeg')) else 'image/png'
                    base64_url = f"data:{mime_type};base64,{base64_str}"
                    processed_img_list.append(base64_url)
                    successful_conversions += 1
                    print(f"已将图片转换为base64格式: {img_url}")
                else:
                    print(f"转换后,base64_str无效，已跳过：{img_url} ")
                    continue

            except Exception as e:
                print(f"图片转base64失败: {img_url}\n{e}")
                continue
        else:
            processed_img_list.append(img_url)

    # 只有当所有有效图片都转换失败时，才抛出异常
    if total_valid_images > 0 and successful_conversions == 0:
        raise Exception(f"图片下载或转换base64失败，url：{img_list}")

    return processed_img_list


def prepare_request_data(platform: str, messages: Any, message_info: Optional[Dict]) -> Tuple[Any, Dict]:
    """准备请求数据

    Args:
        platform: 平台名称
        messages: 请求消息对象
        message_info: 消息信息字典，包含系统提示词、用户文本、是否包含图片和图片列表等

    Returns:
        Tuple[Any, Dict]: (更新后的消息对象, 消息配置字典)
    """
    message_config = {"user_text": "", "include_img": False, "img_list": []}

    if message_info is not None:
        message_config.update(
            {
                "user_text": message_info["user_text"],
                "include_img": message_info.get("include_img", False),
                "img_list": message_info.get("img_list", []),
            }
        )

        system_prompt = message_info["system_prompt"]
        if system_prompt:
            message_config["system_prompt"] = system_prompt

        messages = prepare_messages(
            platform,
            message_config.get("system_prompt", ""),
            message_config["user_text"],
            message_config["include_img"],
            message_config["img_list"],
        )
    return messages, message_config



import regex


# 判断字符串中是否包含汉字
def contains_chinese(text, simple_check=True):
    """
    判断字符串text中是否包含所有汉字（含扩展区）。

    参数:
        text: 要检查的字符串
        simple_check: 是否进行简要判断，默认为True。
                    - True时返回布尔值，表示是否包含汉字
                    - False时返回中文字符的数量

    返回:
        简要模式(True): 布尔值，表示是否包含汉字
        详细模式(False): 整数，表示中文字符的数量
    """
    # 汉字的 Unicode 脚本是 Han
    if simple_check:
        return bool(regex.search(r'\p{IsHan}', text))
    else:
        # 查找所有汉字并返回数量
        matches = regex.findall(r'\p{IsHan}', text)
        chinese_count = len(matches)
        if chinese_count > 0:
            print(f"发现中文字符: {repr(''.join(matches))} (数量: {chinese_count})")
            # print(f"字符详情: {[(i, repr(char), ord(char)) for i, char in enumerate(matches)]}")
        return chinese_count


# 移除字符串中的所有汉字（含扩展区）
def remove_chinese(text):
    """
    移除字符串text中的所有汉字（含扩展区）。
    """
    return regex.sub(r'\p{IsHan}+', '', text)


# 判断字符串中是否包含特殊符号
def contains_special_symbols(text, simple_check=True, support_multilingual=True):
    """
    判断字符串text中是否包含特殊符号（注意：此方法不能用于判断HTML中的特殊符号）。

    参数:
        text: 要检查的字符串
        simple_check: 是否进行简要判断，默认为True。
                    - True时返回布尔值，表示是否包含特殊符号
                    - False时返回特殊符号的数量
        support_multilingual: 是否支持多语言字符，默认为True。
                           - True时保留所有语言的字母字符（默认，支持多语言，如法语等）
                           - False时仅保留英文字母、数字和下划线

    返回:
        简要模式(True): 布尔值，表示是否包含特殊符号
        详细模式(False): 整数，表示特殊符号的数量
    """
    # 定义特殊符号的正则表达式模式
    if support_multilingual:
        # 支持多语言（默认）：使用Unicode属性匹配所有字母字符
        # 排除：.,!?;:'"()[]{}<>-_=+*/\|@#$%^&`~
        pattern = r'[^\p{L}\p{N}\s.,!?;:\'"()\[\]{}<>\-_=+*/\\|@#$%^&`~]'
    else:
        # 仅支持英语：\w只匹配英文字母、数字和下划线
        # 排除：.,!?;:'"()[]{}<>-_=+*/\|@#$%^&`~
        pattern = r'[^\w\s.,!?;:\'"()\[\]{}<>\-_=+*/\\|@#$%^&`~]'

    if simple_check:
        return bool(regex.search(pattern, text))
    else:
        # 查找所有特殊符号并返回数量
        matches = regex.findall(pattern, text)
        print(f"发现特殊符号: {''.join(matches)}")
        return len(matches)


# 移除字符串中的所有特殊符号
def remove_special_symbols(text, support_multilingual=True):
    """
    移除字符串text中的所有特殊符号（注意：此方法不能用于移除HTML中的特殊符号）。

    参数:
        text: 要处理的字符串
        support_multilingual: 是否支持多语言字符，默认为True。
                           - True时保留所有语言的字母字符（默认，支持多语言，如法语等）
                           - False时仅保留英文字母、数字和下划线
    """
    # 定义特殊符号的正则表达式模式
    if support_multilingual:
        # 支持多语言（默认）：使用Unicode属性匹配所有字母字符
        # 排除：.,!?;:'"()[]{}<>-_=+*/\|@#$%^&`~
        pattern = r'[^\p{L}\p{N}\s.,!?;:\'"()\[\]{}<>\-_=+*/\\|@#$%^&`~]'
    else:
        # 仅支持英语：\w只匹配英文字母、数字和下划线
        # 排除：.,!?;:'"()[]{}<>-_=+*/\|@#$%^&`~
        pattern = r'[^\w\s.,!?;:\'"()\[\]{}<>\-_=+*/\\|@#$%^&`~]'
    return regex.sub(pattern, '', text)



"""
重试处理组件
负责处理API请求的重试逻辑、错误处理和异常恢复
"""

from funcguard import print_block, print_line, time_wait
from typing import Dict, Tuple, Any
from ..message import rebuild_messages_single_image, convert_images_to_base64
from .retry_config import (
    IMAGE_DOWNLOAD_ERROR_KEYWORDS,
    DEFAULT_RETRY_KEYWORDS,
    MIN_LIMIT_ERROR_KEYWORDS,
    DEFAULT_RETRY_API_KEYWORDS,
)


class RetryHandler:
    """处理API请求重试逻辑的组件"""

    def __init__(self, platform: str):
        self.platform = platform
        # 获取全局图片缓存
        self.image_cache = None
        try:
            # 注意：必须在这里导入，避免循环引用
            # dispatcher.py 导入 message.convert_to_json，而 utils.retry_handler 又导入 dispatcher.ModelDispatcher
            # 如果在模块级别导入会造成循环引用错误
            from ..dispatcher import ModelDispatcher

            self.image_cache = ModelDispatcher.get_image_cache()
        except ImportError:
            pass

    def _build_error_message(self, error_data: Dict, original_exception: Exception) -> str:
        """构建错误消息字符串

        参数:
            error_data: 包含错误信息的数据字典
            original_exception: 原始异常对象，用于备用错误消息

        返回:
            格式化后的错误消息字符串
        """
        error = error_data.get("error", {})
        message = error_data.get("message", "")

        # 如果没有message，尝试从error中获取
        if not message:
            message = error.get("message", str(original_exception))

        metadata = error.get("metadata", {})  # openrouter
        raw = metadata.get("raw", "")
        provider_name = metadata.get("provider_name", "")

        # 构建基础错误消息
        error_parts = [f"message: {message}"]

        # 只在有值时添加provider信息
        if provider_name:
            error_parts.append(f"provider: {provider_name}")

        # 只在有值时添加detail信息
        if raw:
            error_parts.append(f"detail: {raw}")

        return " , ".join(error_parts)

    def extract_error_message(self, e: Exception) -> str:
        """提取错误信息"""
        error_message = str(e)
        response = getattr(e, 'response', None)

        # 如果异常对象本身就是response（来自handle_response的直接抛出）
        if hasattr(e, 'model_dump'):
            response = e

        if response:
            try:
                if hasattr(response, 'model_dump'):  # 兼容 openai
                    res = response.model_dump()
                    error_message = self._build_error_message(res, e)

                # hasattr 检查 response 是否有 json 方法
                elif hasattr(response, 'json'):
                    res = response.json()
                    # 判断 res 是否是列表
                    if isinstance(res, list):
                        res = res[0]
                    error_message = self._build_error_message(res, e)

            except (AttributeError, ValueError):
                error_message = str(e)

        return error_message

    def should_retry_for_rate_limit(self, error_message: str) -> bool:
        """判断是否因为限流而重试"""
        return any(keyword in error_message for keyword in DEFAULT_RETRY_KEYWORDS)

    def should_retry_for_image_error(self, error_message: str, message_config: Dict) -> bool:
        """判断是否因为图片错误而重试"""
        image_errors = ["输入图片数量超过限制", "图片输入格式/解析错误"]
        return any(error in error_message for error in image_errors) and message_config["include_img"]

    def handle_rate_limit_error(
        self, error_message: str, api_retry_count: int, messages: Any, message_config: Dict
    ) -> Tuple[bool, Any]:
        """处理限流错误

        参数:
            error_message: 错误信息字符串
            api_retry_count: 当前API重试次数（从0开始计数）
            messages: 请求消息对象
            message_config: 消息配置数据字典

        返回:
            Tuple[bool, Any]: (是否继续重试, 更新后的messages对象)
        """
        print(f"请求被限流 或者 网络连接失败，正在第 {api_retry_count + 1} 次重试……")
        # 如果图片：下载或读取 出现问题
        if any(keyword in error_message for keyword in IMAGE_DOWNLOAD_ERROR_KEYWORDS) and message_config["include_img"]:

            img_list = message_config["img_list"]
            print(f"img_list: {img_list}")

            # 第3次尝试时，检测图片格式并进行base64转换
            if api_retry_count == 1 and img_list:
                # print(f"尝试将图片转换为base64格式...")
                img_list = convert_images_to_base64(img_list, self.image_cache)
                # 更新message_config，确保转换后的base64图片在后续重试中继续使用
                # message_config["img_list"] = img_list

            messages = rebuild_messages_single_image(
                self.platform,
                message_config["system_prompt"],
                message_config["user_text"],
                reject_single_image=False,
                img_list=img_list,
            )
        else:
            if any(keyword in error_message for keyword in MIN_LIMIT_ERROR_KEYWORDS):
                time_wait(60 * (api_retry_count + 1))  # 等待一段时间后重试
            else:
                time_wait(10 * (api_retry_count + 1))  # 等待一段时间后重试

        return True, messages

    def handle_image_error(self, messages: Any, message_config: Dict) -> Tuple[bool, Any]:
        """处理图片相关错误

        参数:
            messages: 请求消息对象
            message_config: 消息配置数据字典

        返回:
            Tuple[bool, Any]: (是否继续重试, 更新后的messages对象)
        """

        print("输入图片数量超过限制 或 图片输入格式/解析错误，正在（ 限制图片数量 = 1 ）然后重试...")

        # 只使用第一张图片，并更新message_config
        # if message_config["img_list"]:
        #     message_config["img_list"] = [message_config["img_list"][0]]

        messages = rebuild_messages_single_image(
            self.platform,
            message_config["system_prompt"],
            message_config["user_text"],
            reject_single_image=True,
            img_list=message_config["img_list"],
        )

        return True, messages

    def handle_exception(
        self, e: Exception, api_retry_count: int, messages: Any, message_config: Dict, platform: str, model_name: str
    ) -> Tuple[bool, Any, bool]:
        """处理异常和重试逻辑

        参数:
            e: 异常对象
            api_retry_count: 当前API重试次数（从0开始计数）
            messages: 请求消息对象
            message_config: 消息配置数据字典
            platform: 云服务商平台名称
            model_name: 模型名称

        返回:
            Tuple[bool, Any, bool]: (是否继续重试, 更新后的messages对象, 是否需要切换API密钥)
        """
        # 当前的模型信息
        base_model_info = f"Model {platform} : {model_name}"
        # print(e)

        # 获取错误信息
        error_message = self.extract_error_message(e)

        # 判断是否应该重试
        if self.should_retry_for_rate_limit(error_message):
            print_line()
            print(base_model_info)
            should_retry, updated_messages = self.handle_rate_limit_error(
                error_message, api_retry_count, messages, message_config
            )
            return should_retry, updated_messages, False

        elif self.should_retry_for_image_error(error_message, message_config):
            print_line()
            print(base_model_info)
            should_retry, updated_messages = self.handle_image_error(messages, message_config)
            return should_retry, updated_messages, False

        elif any(keyword in error_message for keyword in DEFAULT_RETRY_API_KEYWORDS):
            print_line()
            print(base_model_info)
            print("模型每日请求超过限制 或 免费额度已用完")
            return True, messages, True  # 需要重试且需要切换API密钥

        else:
            print_line()
            if error_message:
                title = "已提取到报错信息，但未匹配到任何重试场景:"
                print(title)
                print(error_message)
            else:
                title = "注意：未提取到报错信息！"
                print(title)

            # 直接重新抛出原始异常，保持异常对象的完整性
            # print(f"其他异常错误：{e}")
            raise e



"""
类目处理工具函数
"""

from typing import Any, Dict, List, Optional
from ....dispatcher import ModelDispatcher
from ....dispatcher_control import dispatcher_with_repair


def standardize_category_format(cat_data: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    """标准化类目数据格式，统一使用cat_id和cat_name键名"""
    if not cat_data:
        return []

    # 处理字符串类型的情况（可能是从JSON解析失败后的字符串）
    if isinstance(cat_data, str):
        print(f"警告: standardize_category_format接收到字符串而非列表: {cat_data[:50]}...")
        return []

    first_item = cat_data[0]
    if isinstance(first_item, dict):
        if 'value' in first_item and 'label' in first_item:
            return [{'cat_id': item['value'], 'cat_name': item['label']} for item in cat_data]
        elif 'cat_id' in first_item and 'cat_name' in first_item:
            return cat_data

    return cat_data


def extr_cat_tree(cat_tree, level=1, level_1_names=None, level_2_names=None):
    """根据指定的 level 参数提取对应的层级信息"""
    cat_tree = [node for node in cat_tree if node["label"] != "最近添加"]
    result = []

    if level_1_names is not None:
        level_1_names = [cat["cat_name"] for cat in level_1_names]

    level_2_filters = {}
    if level == 3 and level_2_names:
        level_2_names = [cat["cat_name"] for cat in level_2_names]
        for level_2_path in level_2_names:
            parts = level_2_path.split(" > ")
            if len(parts) == 2:
                level_1_name = parts[0]
                level_2_name = parts[1]
                if level_1_name not in level_2_filters:
                    level_2_filters[level_1_name] = set()
                level_2_filters[level_1_name].add(level_2_name)

        if level_1_names is None and level_2_names is not None:
            level_1_names = list(level_2_filters.keys())

    if level == 1:
        unique_values = list({node['value'] for node in cat_tree})
        result = [{'cat_id': value, 'cat_name': value} for value in unique_values]

    elif level == 2:
        for level_1_node in cat_tree:
            if level_1_names is None or level_1_node['value'] in level_1_names:
                children = level_1_node.get('children', [])
                if children:
                    for level_2_node in children:
                        cat_id = f"{level_1_node['value']} > {level_2_node['value']}"
                        cat_name = f"{level_1_node['label']} > {level_2_node['label']}"
                        result_item = {'cat_id': cat_id, 'cat_name': cat_name}
                        if result_item not in result:
                            result.append(result_item)

    elif level == 3:
        for level_1_node in cat_tree:
            if level_1_names is None or level_1_node['value'] in level_2_filters:
                for level_2_node in level_1_node.get('children', []):
                    should_include = level_2_names is None or (
                        level_1_node['value'] in level_2_filters
                        and level_2_node['value'] in level_2_filters[level_1_node['value']]
                    )

                    if should_include:
                        level_3_children = level_2_node.get('children', [])
                        if level_3_children:
                            for level_3_node in level_3_children:
                                level_label_3 = level_3_node['label']
                                cat_id_path = (
                                    f"{level_1_node['value']} > {level_2_node['value']} > {level_3_node['value']}"
                                )
                                cat_name_path = f"{level_1_node['label']} > {level_2_node['label']} > {level_label_3}"
                                result.append({'cat_id': cat_id_path, 'cat_name': cat_name_path})

    return result


def get_category_depth(cat_tree: Any) -> int:
    """检测类目树的最大深度"""
    max_depth = 1

    for level_1_node in cat_tree:
        level_2_children = level_1_node.get('children', [])
        if level_2_children:
            max_depth = max(max_depth, 2)
            for level_2_node in level_2_children:
                level_3_children = level_2_node.get('children', [])
                if level_3_children:
                    max_depth = max(max_depth, 3)
                    break
            if max_depth == 3:
                break

    return max_depth


def prepare_category_data(cat_tree: Any, max_depth: int = 3) -> Any:
    """准备类目数据，根据深度标准化或提取"""
    target_depth = min(get_category_depth(cat_tree), max_depth)

    if target_depth == 1:
        return standardize_category_format(cat_tree)
    else:
        return extr_cat_tree(cat_tree, level=target_depth)


def create_message_info(
    title: str, category_all: Any, system_prompt: str, image_url: str = ""
) -> tuple[Dict[str, Any], str]:
    """创建消息信息和用户文本"""
    user_text = f"商品标题:{title},可选类目:{category_all}"

    message_info = {
        "user_text": user_text,
        "system_prompt": system_prompt,
        "include_img": bool(image_url),
        "img_list": [image_url] if image_url else [],
    }

    return message_info, user_text


def execute_prediction(
    dispatcher: ModelDispatcher,
    message_info: Dict[str, Any],
    group_name: str,
    validate_func: Optional[Any] = None,
    fix_json_config: dict = {},
) -> List[Dict[str, Any]]:
    """执行预测并返回结果"""
    predict_results, _ = dispatcher_with_repair(
        dispatcher,
        message_info,
        group_name,
        validate_func=validate_func,
        fix_json_config=fix_json_config,
    )
    
    # 检查预测结果是否为空
    if not predict_results:
        print("警告: 预测结果为空列表")
    
    return predict_results


# 匹配召回策略
def match_recall(
    category_all: list,
    words: list,
) -> List[Dict[str, Any]]:
    '''根据商品标题匹配召回类目
    Args:
        category_all: 标准化后的所有类目列表，
            示例：[{'cat_id': '……', 'cat_name': '……'}]
        words: 商品标题分词后的单词列表
    Returns:
        匹配的类目列表，示例：[{'cat_id': '……', 'cat_name': '……'}]
    '''

    matched_results = []
    for category in category_all:
        cat_name = category["cat_name"]
        for word in words:
            if word.lower() in cat_name.lower():
                matched_results.append(category)
                break
    print(f"通过split分割，匹配到 {len(matched_results)} 个类目")
    return matched_results


def match_recall_merge(
    matched_results: List[Dict[str, Any]],
    user_suggest_cats: List[Dict[str, Any]],
) -> List[Dict[str, Any]]:
    """合并匹配结果和用户建议类目，并去重

    Args:
        matched_results: 匹配的类目列表
        user_suggest_cats: 用户建议的类目列表

    Returns:
        去重后的类目列表
    """

    if not user_suggest_cats:
        return matched_results

    category_recall = matched_results + user_suggest_cats
    # 去重后的类目
    seen_ids = set()
    category_all = []
    for cat in category_recall:
        cat_id = cat.get("cat_id", "")
        if cat_id and cat_id not in seen_ids:
            seen_ids.add(cat_id)
            category_all.append(cat)
    print(f"通过 match_recall_merge 合并，总计召回类目：{len(category_all)}")

    return category_all



import json
import pandas as pd
from typing import Optional, Union, Any, Tuple

from ollama import chat
from openai import OpenAI
from zhipuai import ZhipuAI

from .utils.retry_handler import RetryHandler
from .message import prepare_request_data
from funcguard.printer import print_line
from funcguard.core import timeout_handler


# 定义基类
def process_stream_response(response):
    result = ""
    for chunk in response:
        result += chunk.choices[0].delta.content
    # 思考：reasoning_chunk = chunk.choices[0].delta.reasoning_content
    # 回答：answer_chunk = chunk.choices[0].delta.content
    return result


class BaseClient:
    def __init__(self, platform: str, model_name: str):
        # 初始化模型参数
        self.model_name = model_name
        self.platform = platform  # 云服务商平台，由子类传入
        self.temperature = 0.4
        self.top_p = 0.1
        self.stream = False  # 是否流式输出，默认为 False，可选为 True
        self.stream_real = False  # 是否真的流式输出
        self.client: Optional[Union[OpenAI, ZhipuAI]] = None  # 由子类初始化
        self.extra_body = {}  # 额外的参数

        # 初始化重试处理器
        self.retry_handler = RetryHandler(self.platform)

    def send_message(self, messages, message_info=None):
        """发送消息的主方法"""

        # 准备请求数据
        messages, request_data = prepare_request_data(self.platform, messages, message_info)

        # 执行重试逻辑
        max_retries = 4  # 考虑到 需要切换api key , 以及可能面临图片异常，设置为 4 比较合理
        api_retry_count = 0

        while api_retry_count < max_retries:
            try:
                # 创建聊天完成请求
                response = timeout_handler(self._create_chat_completion, args=(messages,), execution_timeout=180)

                # 处理响应
                result, total_tokens = self._handle_response(response, self.stream, self.stream_real)
                return result, total_tokens

            except Exception as e:
                # 处理异常和重试逻辑
                should_retry, messages, should_switch_key = self.retry_handler.handle_exception(
                    e, api_retry_count, messages, request_data, self.platform, self.model_name
                )

                if should_switch_key:
                    # 切换API密钥
                    if not self.switch_api_key():
                        print("所有API密钥都已用尽！")
                        raise Exception('API_KEY_EXHAUSTED')

                if should_retry:
                    api_retry_count += 1
                    continue
                else:
                    # 直接重新抛出原始异常，保持异常对象的完整性
                    raise e

        error_message = f"api_retry 达到最大重试次数：{max_retries}"
        print(error_message)
        raise Exception(error_message)

    def _create_chat_completion(self, messages):
        """创建聊天完成请求"""
        if self.client is None:
            raise RuntimeError("客户端未初始化")
        if self.platform == "ollama":
            # https://docs.ollama.com/api#generate-request-with-options
            return chat(
                messages=messages,
                model=self.model_name,
                options={
                    "temperature": self.temperature,
                    "top_p": self.top_p,
                },
                **self.extra_body,  # 传递额外的参数
            )
        else:
            return self.client.chat.completions.create(
                messages=messages,  # type: ignore
                model=self.model_name,
                temperature=self.temperature,
                top_p=self.top_p,
                stream=self.stream,
                **self.extra_body,  # 传递额外的参数
            )

    def _handle_response(
        self,
        response: Any,
        stream: bool,
        stream_real: bool,
    ) -> Tuple[Any, int]:
        """处理响应"""

        total_tokens = 0

        if stream:
            if stream_real:
                result = response
            else:
                result = timeout_handler(process_stream_response, args=(response,), execution_timeout=180)
        else:
            if self.platform == "ollama":
                result = response.message.content
                total_tokens = 0
            else:
                # 非流式响应，直接获取 content
                if isinstance(response, str):
                    raise Exception(f"response 类型是字符串！response: {response}")

                if hasattr(response, 'choices') and response.choices:
                    try:
                        result = response.choices[0].message.content
                    except:
                        # self.platform == "gitcode"，部分情况适用
                        result = response.choices[0].delta["content"]

                    total_tokens = response.usage.total_tokens

                else:
                    # 如果没有choices，检查response是否为异常对象，如果不是则转换为异常
                    print(f"原始响应中没有choices：{response}")
                    if isinstance(response, BaseException):
                        raise response
                    else:
                        raise Exception(response)

        return result, total_tokens


# 定义 BaseOpenai 类
class BaseOpenai(BaseClient):

    def __init__(self, platform, base_url, api_keys, model_name, stream=False, stream_real=False, extra_body=None):
        super().__init__(platform, model_name)
        self.base_url = base_url
        self.api_keys = api_keys
        self.request_timeout = 150  # 新增：API 请求超时时间
        self.max_retries = 3  # 新增：API 请求超时时间

        self.platform = platform
        self.stream = stream
        self.stream_real = stream_real

        # 配置 extra_body 参数
        if extra_body is not None:
            self.extra_body = extra_body

        # 初始化客户端
        self._init_client()

    def _init_client(self):
        """初始化客户端，使用第1个密钥"""
        if not self.api_keys:
            raise Exception("没有可用的API密钥")
        self.api_key = self.api_keys[0]
        if self.platform == "zhipu":
            self.client = ZhipuAI(api_key=self.api_key, timeout=self.request_timeout, max_retries=self.max_retries)
        else:
            self.client = OpenAI(
                api_key=self.api_key, base_url=self.base_url, timeout=self.request_timeout, max_retries=self.max_retries
            )

    def switch_api_key(self):
        """切换API密钥并重新初始化客户端"""
        api_keys_num = len(self.api_keys)
        if api_keys_num >= 2:
            self.api_keys.pop(0)  # 移除第一个密钥
            print(f"移除已用完的密钥，剩余 {api_keys_num-1} 个密钥")
            self._init_client()
            print_line()
            return True
        return False

    def models_df(self):
        if self.client is None:
            raise RuntimeError("客户端未初始化")

        # 获取模型列表
        models_page = self.client.models.list()

        # 提取模型数据
        models_data = [
            {
                'id': model.id,
                'created': model.created,
            }
            for model in models_page.data
        ]

        # 创建DataFrame
        df = pd.DataFrame(models_data)
        # 将created时间戳转换为日期
        df['created_date'] = pd.to_datetime(df['created'], unit='s').dt.strftime('%Y-%m-%d')
        # 移除旧的created列
        df = df.drop('created', axis=1)  # axis=1表示删除列
        # 按照created_date降序排序
        df = df.sort_values('created_date', ascending=False).reset_index(drop=True)

        return df



from llmakits.dispatcher import ModelDispatcher
from ..validators.html_validator import validate_html_fix
from ..validators.string_validator import contains_chinese, remove_chinese
from ...message import extract_field


def generate_html(
    dispatcher: ModelDispatcher,
    product_info: str,
    generate_prompt: str,
    fix_prompt: str,
    generate_group: str,
    fix_group: str,
    allowed_tags: set[str],
    allow_chinese: bool = False,
) -> str:
    """
    生成HTML字符串
    :param dispatcher: 模型调度器
    :param product_info: 产品信息
    :param generate_prompt: 生成HTML提示词
    :param fix_prompt: 修复HTML提示词
    :param generate_group: 生成模型组名
    :param fix_group: 修复模型组名
    :param allowed_tags: 允许的HTML标签集合
    :param allow_chinese: 是否允许HTML中包含中文，默认为True
    :return: 生成的HTML字符串
    """

    def validate_func(return_message: str):
        html_string = extract_field(return_message, "html")
        if not allow_chinese:
            # 检查中文字符的数量
            chinese_count = contains_chinese(html_string, simple_check=False)
            if chinese_count > 0:
                if chinese_count > 5:
                    return False, None
                else:
                    print(f"移除中文字符……")
                    html_string = remove_chinese(html_string)
        des_html = validate_html_fix(dispatcher, html_string, allowed_tags, fix_group, fix_prompt)  # type: ignore
        return True, des_html

    message_info = {"system_prompt": generate_prompt, "user_text": product_info}
    des_html, _ = dispatcher.execute_with_group(  # pyright: ignore[reportAssignmentType]
        message_info, generate_group, format_json=True, validate_func=validate_func
    )

    return des_html



"""
类目验证功能
"""

from typing import Any, Callable, Dict, List, Tuple

from ...validators.value_validator import validate_dict
from .utils import standardize_category_format


def create_category_validate_func(category_all: List[Dict[str, Any]]) -> Callable[[str], Tuple[bool, Any]]:
    """
    创建类目验证函数，用于全部类目一起预测的场景

    Args:
        category_all: 所有可选类目列表

    Returns:
        验证函数
    """

    def validate_func(return_message: str) -> Tuple[bool, Any]:
        """
        验证函数：验证返回的类目是否在可选范围内

        Args:
            return_message: LLM返回的消息

        Returns:
            (验证是否通过, 验证通过的值列表)
        """

        if not return_message:
            return False, None

        # 标准化格式
        standardized_message = standardize_category_format(return_message)  # pyright: ignore[reportArgumentType]

        # 验证每个类目
        predict_results = []
        for category in standardized_message:
            if isinstance(category, dict):
                search_data = {"cat_id": category.get("cat_id", ""), "cat_name": category.get("cat_name", "")}
                value = validate_dict(category_all, search_data)
                if value:
                    predict_results.append(value)

        # 所有类目都验证通过
        if predict_results:
            return True, predict_results
        else:
            # 如果标准化后的消息不为空，但没有验证通过的类目，则验证失败
            if standardized_message:
                print("警告: 预测结果中，没有任何一个验证通过，以下是标准化后的预测结果（standardized_message）: ")
                print(standardized_message)
            return False, predict_results

    return validate_func
